@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: board_properties.c,v 1.13 2004/01/11 23:18:58 fuyu Exp $
+ * $Id: board_properties.c,v 1.14 2004/01/13 02:42:44 fuyu Exp $
  */
 
 #include "config.h"
@@ -54,6 +54,7 @@ typedef struct _DialogStates
 
   GtkEntry *board_name_entry;
   GtkEntry *board_url_entry;
+  GtkToggleButton *update_states_button;
 
   GtkToggleButton *use_ochusha_default_button;
   GtkToggleButton *show_mailto_button;
@@ -414,6 +415,12 @@ apply_current_states(GtkWidget *dialog, DialogStates *states)
 	  info->a_bone_by_content_pattern = NULL;
 	}
     }
+  else
+    {
+      if (states->update_states_button != NULL)
+	info->use_static_threadlist
+	  = !gtk_toggle_button_get_active(states->update_states_button);
+    }
 
   if (gtk_toggle_button_get_active(states->use_as_default_button))
     {
@@ -497,12 +504,16 @@ build_board_info_frame(OchushaApplication *application,
   GtkWidget *table;
   GtkWidget *tmp_widget;
   GtkWidget *board_frame = gtk_frame_new(NULL);
+  GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
 
   gtk_widget_show(board_frame);
 
   table = gtk_table_new(2, 5, FALSE);
   gtk_widget_show(table);
-  gtk_container_add(GTK_CONTAINER(board_frame), table);
+  gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
+  gtk_widget_show(vbox);
+
+  gtk_container_add(GTK_CONTAINER(board_frame), vbox);
 
   if (!IS_VIRTUAL_BOARD(board))
     {
@@ -557,7 +568,8 @@ build_board_info_frame(OchushaApplication *application,
 
       tmp_widget = gtk_entry_new();
       gtk_entry_set_text(GTK_ENTRY(tmp_widget), board->id);
-      if (strcmp(board->id, "favorites") == 0)
+      if (strcmp(board->id, "favorites") == 0
+	  || strcmp(board->id, "all-threads") == 0)
 	{
 	  gtk_editable_set_editable(GTK_EDITABLE(tmp_widget), FALSE);
 	  GTK_WIDGET_UNSET_FLAGS(tmp_widget, GTK_CAN_FOCUS);
@@ -571,6 +583,17 @@ build_board_info_frame(OchushaApplication *application,
 
       gtk_widget_show(tmp_widget);
       gtk_table_attach_defaults(GTK_TABLE(table), tmp_widget, 1, 5, 1, 2);
+
+      if (strcmp(board->id, "all-threads") != 0)
+	{
+	  tmp_widget
+	    = gtk_check_button_new_with_label(_("Update States When Opened."));
+	  gtk_widget_show(tmp_widget);
+	  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
+	  states->update_states_button = GTK_TOGGLE_BUTTON(tmp_widget);
+	  gtk_toggle_button_set_active(states->update_states_button,
+				       !states->info->use_static_threadlist);
+	}
     }
 
   return board_frame;
