@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_ui.c,v 1.69 2004/01/11 22:34:41 fuyu Exp $
+ * $Id: boardlist_ui.c,v 1.71 2004/01/13 02:41:52 fuyu Exp $
  */
 
 #include "config.h"
@@ -90,6 +90,8 @@ static gboolean kill_board_cb(BoardlistView *view,
 			      OchushaApplication *application);
 static void copy_board_url_cb(BoardlistView *view, OchushaBulletinBoard *board,
 			      OchushaApplication *application);
+static void open_properties_cb(BoardlistView *view, GObject *object,
+			       OchushaApplication *application);
 static void uri_dropped_cb(BoardlistView *view, OchushaBoardCategory *category,
 			   OchushaBulletinBoard *board, const char *uri,
 			   GtkTreeViewDropPosition pos,
@@ -245,16 +247,30 @@ ensure_my_folder(OchushaApplication *application)
 				       application->favorites);
     }
 
-#if 0
+  if (application->favorites->thread_list == NULL)
+    ochusha_bulletin_board_read_threadlist_xml(application->favorites,
+					       &application->config,
+					       application->session_subdir,
+					       FALSE);
+
   if (application->all_threads == NULL)
     {
+      BulletinBoardGUIInfo *board_info;
       application->all_threads = virtual_board_new(_("All Threads Read"),
 						   "http:///all-threads",
 						   application);
+      board_info = ensure_bulletin_board_info(application->all_threads,
+					      application);
+      board_info->use_static_threadlist = TRUE;
       ochusha_board_category_add_board(application->my_folder,
 				       application->all_threads);
     }
-#endif
+
+  if (application->all_threads->thread_list == NULL)
+    ochusha_bulletin_board_read_threadlist_xml(application->all_threads,
+					       &application->config,
+					       application->session_subdir,
+					       FALSE);
 }
 
 
@@ -289,6 +305,8 @@ initialize_boardlist(OchushaApplication *application)
 		   G_CALLBACK(kill_board_cb), application);
   g_signal_connect(G_OBJECT(application->boardlist_view), "copy_board_url",
 		   G_CALLBACK(copy_board_url_cb), application);
+  g_signal_connect(G_OBJECT(application->boardlist_view), "open_properties",
+		   G_CALLBACK(open_properties_cb), application);
 
   g_signal_connect(G_OBJECT(application->boardlist_view), "uri_dropped",
 		   G_CALLBACK(uri_dropped_cb), application);
@@ -530,13 +548,23 @@ finalize_boardlist(OchushaApplication *application)
 	fprintf(stderr, "Couldn't write boardlist.xml.\n");
     }
 
+  /* XXX: Ĥɲä򥵥ݡȤ顢ͤɬפ */
   if (IS_VIRTUAL_BOARD(application->favorites))
     {
       VirtualBoard *favorites = VIRTUAL_BOARD(application->favorites);
       g_slist_foreach(favorites->board_list, update_board_list, NULL);
+      virtual_board_clean_up_threadlist(favorites, FALSE);
       write_threadlist(favorites, application);
     }
 
+  if (IS_VIRTUAL_BOARD(application->all_threads))
+    {
+      VirtualBoard *all_threads = VIRTUAL_BOARD(application->all_threads);
+      g_slist_foreach(all_threads->board_list, update_board_list, NULL);
+      virtual_board_clean_up_threadlist(all_threads, TRUE);
+      write_threadlist(all_threads, application);
+    }
+
   g_slist_foreach(board_list, write_threadlist, application);
 }
 
@@ -1111,7 +1139,14 @@ item_view_being_closed_cb(PanedNotebook *paned_notebook,
     }
 
   if (buffer != NULL)
-    OCHU_OBJECT_UNREF(G_OBJECT(buffer));
+    {
+      OCHU_OBJECT_UNREF(G_OBJECT(buffer));
+
+      if (board == application->favorites)
+	virtual_board_clean_up_threadlist(VIRTUAL_BOARD(board), FALSE);
+      else if (board == application->all_threads)
+	virtual_board_clean_up_threadlist(VIRTUAL_BOARD(board), TRUE);
+    }
 
   if (info->message_id != 0)
     {
@@ -1209,10 +1244,8 @@ board_new_cb(OchushaBBSTable *table, int bbs_type, const gchar *name,
 	= virtual_board_new(name, base_url, application);
       if (strcmp(base_url, "http:///favorites") == 0)
 	application->favorites = board;
-#if 0
       else if (strcmp(base_url, "http:///all-threads") == 0)
 	application->all_threads = board;
-#endif
       return board;
     }
   return NULL;	/* ¾ΤϤǤ */
@@ -1255,6 +1288,10 @@ boardlist_read_board_element_cb(OchushaBBSTable *table,
       || info->show_mailto_mode > THREAD_VIEW_MAILTO_MODE_HIDE)
     info->show_mailto_mode = THREAD_VIEW_MAILTO_MODE_DEFAULT;
 
+  info->use_static_threadlist
+    = ochusha_utils_get_attribute_boolean(board_attributes,
+					 "use_static_threadlist");
+
   info->enable_filter = ochusha_utils_get_attribute_boolean(board_attributes,
 							    "enable_filter");
   info->filter.rule = ochusha_utils_get_attribute_string(board_attributes,
@@ -1383,18 +1420,20 @@ boardlist_write_board_element_cb(OchushaBBSTable *table,
 {
   BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board, application);
   OUTPUT_BOARD_ATTRIBUTE_INT(boardlist_xml, info, show_mailto_mode);
-  OUTPUT_BOARD_ATTRIBUTE_BOOLEAN(boardlist_xml, info, enable_filter);
   OUTPUT_BOARD_ATTRIBUTE_STRING(boardlist_xml, info, last_name);
   OUTPUT_BOARD_ATTRIBUTE_STRING(boardlist_xml, info, last_mail);
   if (info->recover_mode)
     OUTPUT_BOARD_ATTRIBUTE_BOOLEAN(boardlist_xml, info, recover_mode);
 
+  OUTPUT_BOARD_ATTRIBUTE_BOOLEAN(boardlist_xml, info, enable_filter);
   if (info->filter.rule != NULL)
     {
       OUTPUT_BOARD_ATTRIBUTE_STRING(boardlist_xml, info, filter.rule);
       OUTPUT_BOARD_ATTRIBUTE_INT(boardlist_xml, info, filter.ignore_threshold);
     }
 
+  OUTPUT_BOARD_ATTRIBUTE_BOOLEAN(boardlist_xml, info, use_static_threadlist);
+
   if (info->board_local_a_bone)
     {
       OUTPUT_BOARD_ATTRIBUTE_BOOLEAN(boardlist_xml, info, board_local_a_bone);
@@ -1520,6 +1559,17 @@ copy_board_url_cb(BoardlistView *view, OchushaBulletinBoard *board,
 }
 
 
+static void
+open_properties_cb(BoardlistView *view, GObject *object,
+		   OchushaApplication *application)
+{
+  if (!OCHUSHA_IS_BULLETIN_BOARD(object))
+    return;
+
+  open_board_properties(application, OCHUSHA_BULLETIN_BOARD(object));
+}
+
+
 static void
 uri_dropped_cb(BoardlistView *view, OchushaBoardCategory *category,
 	       OchushaBulletinBoard *board, const char *uri,
