@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_view.c,v 1.28 2004/01/11 22:34:41 fuyu Exp $
+ * $Id: boardlist_view.c,v 1.29 2004/01/13 02:41:52 fuyu Exp $
  */
 
 #include "config.h"
@@ -126,6 +126,7 @@ enum {
   KILL_CATEGORY_SIGNAL,
   KILL_BOARD_SIGNAL,
   COPY_BOARD_URL_SIGNAL,
+  OPEN_PROPERTIES_SIGNAL,
 
   ADVANCE_VIEW_SIGNAL,
   BACK_VIEW_SIGNAL,
@@ -153,13 +154,14 @@ enum {
   POPUP_MENU_OPEN_SELECTION_WITH_BROWSER,
   POPUP_MENU_TOGGLE_HIDE_SELECTION,
   POPUP_MENU_KILL_SELECTION,
-  POPUP_MENU_COPY_BOARD_URL
+  POPUP_MENU_COPY_BOARD_URL,
+  POPUP_MENU_OPEN_PROPERTIES
 };
 
 
 static GtkTreeViewClass *parent_class = NULL;
 static int boardlist_view_signals[LAST_SIGNAL] =
-  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
+  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
 static GtkItemFactory *popup_menu_item_factory = NULL;
 static GtkItemFactory *popup_menu_item_factory2 = NULL;
 static GQuark node_info_id;
@@ -259,6 +261,20 @@ boardlist_view_class_init(BoardlistViewClass *klass)
 	POPUP_MENU_COPY_BOARD_URL,		/* act */
 	NULL					/* type */
       },
+      {
+	_("/-------"),
+	NULL,
+	NULL,
+	0,
+	(char *)"<Separator>"
+      },
+      {
+	_("/Open Properties"),			/* menu path */
+	NULL,					/* accelerator */
+	popup_menu_callback,			/* callback_func */
+	POPUP_MENU_OPEN_PROPERTIES,		/* act */
+	NULL					/* type */
+      },
     };
   int nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);
 
@@ -320,6 +336,20 @@ boardlist_view_class_init(BoardlistViewClass *klass)
 	POPUP_MENU_COPY_BOARD_URL,		/* act */
 	NULL					/* type */
       },
+      {
+	_("/-------"),
+	NULL,
+	NULL,
+	0,
+	(char *)"<Separator>"
+      },
+      {
+	_("/Open Properties"),			/* menu path */
+	NULL,					/* accelerator */
+	popup_menu_callback2,			/* callback_func */
+	POPUP_MENU_OPEN_PROPERTIES,		/* act */
+	NULL					/* type */
+      },
     };
   int nmenu_items2 = sizeof(menu_items2) / sizeof(menu_items2[0]);
 
@@ -457,6 +487,15 @@ boardlist_view_class_init(BoardlistViewClass *klass)
 		 ochusha_marshal_VOID__OBJECT,
 		 G_TYPE_NONE, 1,
 		 OCHUSHA_TYPE_BULLETIN_BOARD);
+  boardlist_view_signals[OPEN_PROPERTIES_SIGNAL] =
+    g_signal_new("open_properties",
+		 G_TYPE_FROM_CLASS(klass),
+		 G_SIGNAL_RUN_LAST,
+		 G_STRUCT_OFFSET(BoardlistViewClass, open_properties),
+		 NULL, NULL,
+		 ochusha_marshal_VOID__OBJECT,
+		 G_TYPE_NONE, 1,
+		 G_TYPE_OBJECT);
 
   boardlist_view_signals[ADVANCE_VIEW_SIGNAL] =
     g_signal_new("advance_view",
@@ -1164,6 +1203,13 @@ do_on_selection_helper(gpointer list_data, gpointer user_data)
 		      0,
 		      data);
       break;
+
+    case POPUP_MENU_OPEN_PROPERTIES:
+      g_signal_emit(G_OBJECT(args->view),
+		    boardlist_view_signals[OPEN_PROPERTIES_SIGNAL],
+		    0,
+		    data);
+      break;
     }
 
  done:
