@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bulletin_board_ui.c,v 1.112 2004/01/11 22:33:04 fuyu Exp $
+ * $Id: bulletin_board_ui.c,v 1.115 2004/01/13 02:43:30 fuyu Exp $
  */
 
 #include "config.h"
@@ -87,8 +87,14 @@ static void toggle_hide_cb(ThreadlistView *view, OchushaBBSThread *thread,
 			   OchushaApplication *application);
 static void reset_thread_cb(ThreadlistView *view, OchushaBBSThread *thread,
 			    OchushaApplication *application);
+static void super_reset_thread_cb(ThreadlistView *view,
+				  OchushaBBSThread *thread,
+				  OchushaApplication *application);
 static void kill_thread_cb(ThreadlistView *view, OchushaBBSThread *thread,
 			   VirtualBoard *board);
+static void super_kill_thread_cb(ThreadlistView *view,
+				 OchushaBBSThread *thread,
+				 OchushaApplication *application);
 static void copy_thread_url_cb(ThreadlistView *view, OchushaBBSThread *thread,
 			       OchushaApplication *application);
 static void bookmark_thread_cb(ThreadlistView *view, OchushaBBSThread *thread,
@@ -925,16 +931,31 @@ open_bulletin_board(OchushaApplication *application,
 		     G_CALLBACK(close_threadlist_cb), application);
     g_signal_connect(G_OBJECT(view), "toggle_hide",
 		     G_CALLBACK(toggle_hide_cb), application);
-    g_signal_connect(G_OBJECT(view), "reset_thread",
-		     G_CALLBACK(reset_thread_cb), application);
     if (IS_VIRTUAL_BOARD(board))
       {
-	threadlist_view_set_editable(THREADLIST_VIEW(view), TRUE);
-	g_signal_connect(G_OBJECT(view), "kill_thread",
-			 G_CALLBACK(kill_thread_cb), board);
+	if (board != application->all_threads)
+	  {
+	    threadlist_view_set_editable(THREADLIST_VIEW(view), TRUE);
+	    g_signal_connect(G_OBJECT(view), "kill_thread",
+			     G_CALLBACK(kill_thread_cb), board);
+	    g_signal_connect(G_OBJECT(view), "reset_thread",
+			     G_CALLBACK(reset_thread_cb), application);
+	  }
+	else
+	  {
+	    threadlist_view_set_editable(THREADLIST_VIEW(view), TRUE);
+	    g_signal_connect(G_OBJECT(view), "kill_thread",
+			     G_CALLBACK(super_kill_thread_cb), application);
+	    g_signal_connect(G_OBJECT(view), "reset_thread",
+			     G_CALLBACK(super_reset_thread_cb), application);
+	  }
       }
     else
-      threadlist_view_set_editable(THREADLIST_VIEW(view), FALSE);
+      {
+	threadlist_view_set_editable(THREADLIST_VIEW(view), FALSE);
+	g_signal_connect(G_OBJECT(view), "reset_thread",
+			 G_CALLBACK(reset_thread_cb), application);
+      }
     g_signal_connect(G_OBJECT(view), "copy_thread_url",
 		     G_CALLBACK(copy_thread_url_cb), application);
     g_signal_connect(G_OBJECT(view), "bookmark_thread",
@@ -1358,6 +1379,7 @@ open_kako_thread_view(WorkerThread *employee, gpointer job_args)
 typedef struct _EachThreadArgs
 {
   OchushaApplication *application;
+  OchushaBulletinBoard *board;
   BulletinBoardGUIInfo *info;
   ThreadlistView *view;
   int number_of_threads;
@@ -1428,6 +1450,26 @@ each_thread_cb(OchushaBBSThread *thread, gpointer user_data)
   G_FREE(title);
 #endif
 
+  if (args->info->recover_mode)
+    {
+      if (args->board == args->application->all_threads)
+	{
+	  if (ochusha_bbs_thread_get_number_of_responses_read(thread) == 0)
+	    return TRUE;
+	}
+      else
+	{
+	  OCHU_THREADS_ENTER();
+	  if (ochusha_bbs_thread_get_number_of_responses_read(thread) > 0)
+	    bookmark_thread(args->application->all_threads, thread,
+			    args->application);
+	  else
+	    bookmark_remove_thread(args->application->all_threads, thread,
+				   args->application);
+	  OCHU_THREADS_LEAVE();
+	}
+    }
+
   args->number_of_threads++;
 
   if (update_condition)
@@ -1482,6 +1524,7 @@ show_threadlist(WorkerThread *employee, gpointer args)
   EachThreadArgs cb_args =
     {
       application,
+      board,
       info,
       view,
       0,			/* number_of_threads */
@@ -1498,10 +1541,12 @@ show_threadlist(WorkerThread *employee, gpointer args)
   }
 #endif
 
-  ochusha_bulletin_board_read_threadlist_xml(board, &application->config,
-					     application->session_subdir,
-					     info->recover_mode);
-  info->recover_mode = FALSE;
+  if (board->thread_list == NULL || info->recover_mode)
+    {
+      ochusha_bulletin_board_read_threadlist_xml(board, &application->config,
+						 application->session_subdir,
+						 info->recover_mode);
+    }
 
   THREAD_LIST_LOCK
   {
@@ -1588,6 +1633,7 @@ show_threadlist(WorkerThread *employee, gpointer args)
     if (!info->redraw_required)
       {
 	info->is_busy = FALSE;
+	info->recover_mode = FALSE;
 	THREAD_LIST_COND_BROADCAST;
       }
     else
@@ -1813,6 +1859,20 @@ reset_thread_cb(ThreadlistView *view, OchushaBBSThread *thread,
   info->view_flags = 0;
   info->bookmark_response_number = 0;
   update_threadlist_entry_style(application, view, thread);
+  bookmark_remove_thread(application->all_threads, thread, application);
+}
+
+
+static void
+super_reset_thread_cb(ThreadlistView *view, OchushaBBSThread *thread,
+		      OchushaApplication *application)
+{
+  BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
+  ochusha_bbs_thread_remove_cache(thread, &application->config);
+  info->view_flags = 0;
+  info->bookmark_response_number = 0;
+  update_threadlist_entry_style(application, view, thread);
+  virtual_board_remove_thread(VIRTUAL_BOARD(application->all_threads), thread);
 }
 
 
@@ -1824,6 +1884,18 @@ kill_thread_cb(ThreadlistView *view, OchushaBBSThread *thread,
 }
 
 
+static void
+super_kill_thread_cb(ThreadlistView *view, OchushaBBSThread *thread,
+		     OchushaApplication *application)
+{
+  BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
+  ochusha_bbs_thread_remove_cache(thread, &application->config);
+  info->view_flags = 0;
+  info->bookmark_response_number = 0;
+  virtual_board_remove_thread(VIRTUAL_BOARD(application->all_threads), thread);
+}
+
+
 static void
 copy_thread_url_cb(ThreadlistView *view, OchushaBBSThread *thread,
 		   OchushaApplication *application)
@@ -1838,16 +1910,21 @@ bookmark_thread(OchushaBulletinBoard *board, OchushaBBSThread *thread,
 {
   GtkWidget *widget;
   const char *id = ochusha_bbs_thread_get_url(thread);
-  OchushaBBSThread *proxy
-    = ochusha_bulletin_board_lookup_bbs_thread_by_id(board, id);
-  if (proxy != NULL)
-    return;	/* bookmark */
+  OchushaBBSThread *proxy;
+  gboolean bookmarked;
 
   g_return_if_fail(IS_VIRTUAL_BOARD(board));
 
-  proxy = ochusha_bulletin_board_bbs_thread_new(board, id, thread->title);
-  thread_proxy_set_real_thread(THREAD_PROXY(proxy), thread);
-  board->thread_list = g_slist_append(board->thread_list, proxy);
+  proxy = ochusha_bulletin_board_lookup_bbs_thread_by_id(board, id);
+  if (proxy != NULL)
+    bookmarked = TRUE;
+  else
+    {
+      proxy = ochusha_bulletin_board_bbs_thread_new(board, id, thread->title);
+      thread_proxy_set_real_thread(THREAD_PROXY(proxy), thread);
+      board->thread_list = g_slist_append(board->thread_list, proxy);
+      bookmarked = FALSE;
+    }
 
   widget = paned_notebook_get_item_view(PANED_NOTEBOOK(application->contents_window), board);
   if (widget != NULL)
@@ -1859,21 +1936,65 @@ bookmark_thread(OchushaBulletinBoard *board, OchushaBBSThread *thread,
 	  ThreadlistView *favorites_view
 	    = THREADLIST_VIEW(gtk_bin_get_child(GTK_BIN(widget)));
 	  BBSThreadGUIInfo *info = ensure_bbs_thread_info(proxy);
-	  threadlist_view_append_thread(favorites_view,
-					proxy, info->num_shown,
-					(MAX(ochusha_bbs_thread_get_number_of_responses_on_server(proxy),
-					 ochusha_bbs_thread_get_number_of_responses_read(proxy))
-					 - info->bookmark_response_number),
-					info->view_rank, 1,
-					calculate_weight(proxy, info,
-							 1,
-							 application->use_builtin_calculate_weight));
+	  if (bookmarked)
+	    {
+	      threadlist_view_update_thread(favorites_view,
+					    proxy, info->num_shown,
+					    (MAX(ochusha_bbs_thread_get_number_of_responses_on_server(proxy),
+						 ochusha_bbs_thread_get_number_of_responses_read(proxy))
+					     - info->bookmark_response_number),
+					    calculate_weight(proxy, info,
+							     info->view_rank,
+							     application->use_builtin_calculate_weight));
+	    }
+	  else
+	    {
+	      threadlist_view_append_thread(favorites_view,
+					    proxy, info->num_shown,
+					    (MAX(ochusha_bbs_thread_get_number_of_responses_on_server(proxy),
+						 ochusha_bbs_thread_get_number_of_responses_read(proxy))
+					     - info->bookmark_response_number),
+					    info->view_rank, 1,
+					    calculate_weight(proxy, info,
+							     1,
+							     application->use_builtin_calculate_weight));
+	    }
 	  update_threadlist_entry_style(application, favorites_view, proxy);
 	}
     }
 }
 
 
+void
+bookmark_remove_thread(OchushaBulletinBoard *board, OchushaBBSThread *thread,
+		       OchushaApplication *application)
+{
+  GtkWidget *widget;
+  const char *id = ochusha_bbs_thread_get_url(thread);
+  OchushaBBSThread *proxy;
+
+  g_return_if_fail(IS_VIRTUAL_BOARD(board));
+  proxy = ochusha_bulletin_board_lookup_bbs_thread_by_id(board, id);
+  if (proxy == NULL)
+    return;	/* Ѥ⤽äƤʤ */
+
+  virtual_board_remove_thread(VIRTUAL_BOARD(board), proxy);
+
+  widget = paned_notebook_get_item_view(PANED_NOTEBOOK(application->contents_window), board);
+  if (widget != NULL)
+    {
+      /* widgetPanedNotebook */
+      widget = PANED_NOTEBOOK(widget)->selector;
+      if (GTK_IS_SCROLLED_WINDOW(widget))
+	{
+	  ThreadlistView *favorites_view
+	    = THREADLIST_VIEW(gtk_bin_get_child(GTK_BIN(widget)));
+	  threadlist_view_remove_thread(favorites_view, proxy);
+	}
+    }
+}
+
+
 static void
 bookmark_thread_cb(ThreadlistView *view, OchushaBBSThread *thread,
 		   OchushaApplication *application)
