@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: main.c,v 1.14 2003/12/29 08:47:44 fuyu Exp $
+ * $Id: main.c,v 1.15 2003/12/30 15:51:01 fuyu Exp $
  */
 
 #include "config.h"
@@ -82,7 +82,11 @@ main(int argc, char *argv[])
   init_memory_trace();
 #endif
 
+#ifdef REG_POSIX_ENCODING_UTF8
+  reg_set_encoding(REG_POSIX_ENCODING_UTF8);
+#else	/* old style */
   reg_set_encoding(REG_ENCODING_UTF8);
+#endif
 
   application = G_NEW0(OchushaApplication, 1);
   ochusha_config_prepare_home(&application->config);
