@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_ui.c,v 1.77 2004/01/11 22:35:17 fuyu Exp $
+ * $Id: ochusha_ui.c,v 1.80 2004/01/13 02:56:12 fuyu Exp $
  */
 
 #include "config.h"
@@ -76,9 +76,15 @@
 
 #include "ochusha48.h"
 
+#undef NEW_YEAR_SPECIAL
+#undef XMAS_SPECIAL
+
 #include "ochusha-banner.h"
+
+#ifdef NEW_YEAR_SPECIAL
 #include "ochusha-banner1.h"
-#define XMAS_SPECIAL	0
+#endif
+
 #ifdef XMAS_SPECIAL
 #include "ochusha-banner2.h"
 #endif
@@ -166,7 +172,9 @@ static pthread_mutex_t main_ui_lock;
 static GtkWidget *preference_dialog = NULL;
 static GtkWidget *about_dialog = NULL;
 static GdkPixbuf *banner_pixbuf = NULL;
+#ifdef NEW_YEAR_SPECIAL
 static GdkPixbuf *banner1_pixbuf = NULL;
+#endif
 #ifdef XMAS_SPECIAL
 static GdkPixbuf *banner2_pixbuf = NULL;
 #endif
@@ -440,8 +448,10 @@ ochusha_init_application(OchushaApplication *application)
 
   banner_pixbuf
     = gdk_pixbuf_new_from_inline(-1, ochusha_banner_inline, FALSE, NULL);
+#ifdef NEW_YEAR_SPECIAL
   banner1_pixbuf
     = gdk_pixbuf_new_from_inline(-1, ochusha_banner1_inline, FALSE, NULL);
+#endif
 #ifdef XMAS_SPECIAL
   banner2_pixbuf
     = gdk_pixbuf_new_from_inline(-1, ochusha_banner2_inline, FALSE, NULL);
@@ -804,8 +814,12 @@ setup_for_tab_label_animation(OchushaAsyncBuffer *buffer, IconLabel *tab_label)
 		   buffer);
   g_object_set_qdata(G_OBJECT(buffer), tab_label_id, tab_label);
 
+#if 1	/* ɽ֤Ϲˤ褦 */
+  icon_label_set_icon(tab_label, tab_label_icons[TAB_LABEL_ICON_DONE]);
+#else
   if (buffer->fixed)
     icon_label_set_icon(tab_label, tab_label_icons[TAB_LABEL_ICON_DONE]);
+#endif
 }
 
 
@@ -1191,6 +1205,8 @@ write_optional_prefs(gzFile *file, gpointer user_data)
   OchushaApplication *application = (OchushaApplication *)user_data;
   gchar *bookmark_text;
 
+  OUTPUT_CONFIG_ATTRIBUTE_INT(file, application, config_version);
+
   OUTPUT_CONFIG_ATTRIBUTE_INT(file, application, width);
   OUTPUT_CONFIG_ATTRIBUTE_INT(file, application, height);
   OUTPUT_CONFIG_ATTRIBUTE_INT(file, application, boardlist_width);
@@ -1314,6 +1330,7 @@ static void
 configure_application(OchushaApplication *application)
 {
   gboolean succeeded;
+  int version;
 
   ochusha_read_config_xml(&application->config, application->session_subdir,
 			  read_optional_prefs, application);
@@ -1426,11 +1443,14 @@ configure_application(OchushaApplication *application)
       = G_STRDUP(OCHUSHA_DEFAULT_WEB_BROWSER_TEMPLATE);
 
   succeeded = application->shutdown_succeeded;
+  version = application->config_version;
+  application->config_version = OCHUSHA_CURRENT_CONFIG_VERSION;
+
   application->shutdown_succeeded = FALSE;
   ochusha_write_preferences(application);
   application->shutdown_succeeded = succeeded;
 
-  if (!succeeded)
+  if (!succeeded || version != OCHUSHA_CURRENT_CONFIG_VERSION)
     application->recover_mode = TRUE;
   else
     application->recover_mode = FALSE;
@@ -1447,6 +1467,8 @@ read_optional_prefs(GHashTable *pref_attrs, gpointer user_data)
   OchushaApplication *application = (OchushaApplication *)user_data;
   gchar *bookmark_text;
 
+  application->config_version
+    = ochusha_utils_get_attribute_int(pref_attrs, "config_version");
   application->width = ochusha_utils_get_attribute_int(pref_attrs, "width");
 
   application->height = ochusha_utils_get_attribute_int(pref_attrs, "height");
@@ -2933,14 +2955,17 @@ open_about_dialog(OchushaApplication *application)
 	  struct tm tm;
 	  time_t utc = time(NULL);
 	  localtime_r(&utc, &tm);
+	  banner = NULL;
 
+#ifdef NEW_YEAR_SPECIAL
 	  if (tm.tm_year == 104 && tm.tm_yday < 3)
 	    banner = banner1_pixbuf;
+#endif
 #ifdef XMAS_SPECIAL
 	  else if (tm.tm_yday >= 357 && tm.tm_yday <= 358)
 	    banner = banner2_pixbuf;
 #endif
-	  else
+	  if (banner == NULL)
 	    banner = banner_pixbuf;
 	}
 #else
