@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: paned_notebook.c,v 1.14 2003/12/30 21:53:28 fuyu Exp $
+ * $Id: paned_notebook.c,v 1.17 2004/01/11 03:03:58 fuyu Exp $
  */
 
 #include "config.h"
@@ -89,9 +89,9 @@ static void menu_item_activate_cb(GtkMenuItem *menu_item,
 				  PanedNotebook *paned_notebook);
 static void close_item_view_button_cb(gpointer unused,
 				      PanedNotebook *paned_notebook);
-static void go_forward_button_cb(gpointer unused,
-				 PanedNotebook *paned_notebook);
-static void go_back_button_cb(gpointer unused, PanedNotebook *paned_notebook);
+static void go_left_button_cb(gpointer unused, PanedNotebook *paned_notebook);
+static void go_right_button_cb(gpointer unused, PanedNotebook *paned_notebook);
+
 
 GType
 paned_notebook_get_type(void)
@@ -350,6 +350,8 @@ static void
 paned_notebook_init(PanedNotebook *paned_notebook)
 {
   GtkPaned *paned;
+  GtkWidget *tmp_arrow;
+  GtkWidget *tmp_button;
 
   g_return_if_fail(GTK_IS_PANED(paned_notebook));
 
@@ -363,11 +365,23 @@ paned_notebook_init(PanedNotebook *paned_notebook)
 		     GTK_WIDGET(paned_notebook->toolbar),
 		     TRUE, TRUE, 0);
 
+  gtk_toolbar_set_style(paned_notebook->toolbar, GTK_TOOLBAR_ICONS);
+#if 0
+  gtk_toolbar_set_icon_size(paned_notebook->toolbar,
+			    GTK_ICON_SIZE_SMALL_TOOLBAR);
+#endif
+
   paned_notebook->right_toolbar = GTK_TOOLBAR(gtk_toolbar_new());
   gtk_box_pack_end(paned_notebook->toolbar_box,
 		   GTK_WIDGET(paned_notebook->right_toolbar),
 		   FALSE, TRUE, 0);
 
+  gtk_toolbar_set_style(paned_notebook->right_toolbar, GTK_TOOLBAR_ICONS);
+#if 0
+  gtk_toolbar_set_icon_size(paned_notebook->right_toolbar,
+			    GTK_ICON_SIZE_SMALL_TOOLBAR);
+#endif
+
   gtk_box_pack_start(GTK_BOX(paned_notebook->contents_pane),
 		     GTK_WIDGET(paned_notebook->toolbar_box),
 		     FALSE, TRUE, 0);
@@ -383,28 +397,47 @@ paned_notebook_init(PanedNotebook *paned_notebook)
 
   /* MEMO: tooltipΥåpropertyǽˤ٤ */
   gtk_toolbar_insert_stock(paned_notebook->right_toolbar, GTK_STOCK_CLOSE,
-			   _("Close current page"),
+			   _("Close the Current Page"),
 			   "close_current_page",
 			   GTK_SIGNAL_FUNC(close_item_view_button_cb),
 			   paned_notebook,
 			   0);
 
-  gtk_toolbar_insert_stock(paned_notebook->toolbar, GTK_STOCK_GO_FORWARD,
-			   _("Go next page"),
-			   "go_next_page",
-			   GTK_SIGNAL_FUNC(go_forward_button_cb),
-			   paned_notebook,
-			   0);
-  gtk_toolbar_insert_stock(paned_notebook->toolbar, GTK_STOCK_GO_BACK,
-			   _("Go previous page"),
-			   "go_previous_page",
-			   GTK_SIGNAL_FUNC(go_back_button_cb),
-			   paned_notebook,
-			   0);
+  tmp_arrow = gtk_arrow_new(GTK_ARROW_LEFT, GTK_SHADOW_ETCHED_OUT);
+  tmp_button = gtk_button_new();
+  gtk_button_set_relief(GTK_BUTTON(tmp_button), GTK_RELIEF_NONE);
+  gtk_widget_show(tmp_arrow);
+  gtk_container_add(GTK_CONTAINER(tmp_button), tmp_arrow);
+  GTK_WIDGET_UNSET_FLAGS(tmp_button, GTK_CAN_FOCUS);
+  gtk_widget_show(tmp_button);
+  g_signal_connect(G_OBJECT(tmp_button), "clicked",
+		   G_CALLBACK(go_left_button_cb), paned_notebook);
+  gtk_toolbar_append_widget(paned_notebook->toolbar, tmp_button,
+			    _("Go to the Left Page"),
+			    "go_left_page");
+  paned_notebook->left_arrow = tmp_button;
+  gtk_widget_set_sensitive(tmp_button, FALSE);
+
+  tmp_arrow = gtk_arrow_new(GTK_ARROW_RIGHT, GTK_SHADOW_ETCHED_OUT);
+  tmp_button = gtk_button_new();
+  gtk_button_set_relief(GTK_BUTTON(tmp_button), GTK_RELIEF_NONE);
+  gtk_widget_show(tmp_arrow);
+  gtk_container_add(GTK_CONTAINER(tmp_button), tmp_arrow);
+  gtk_widget_show(tmp_button);
+  g_signal_connect(G_OBJECT(tmp_button), "clicked",
+		   G_CALLBACK(go_right_button_cb), paned_notebook);
+  GTK_WIDGET_UNSET_FLAGS(tmp_button, GTK_CAN_FOCUS);
+  gtk_toolbar_append_widget(paned_notebook->toolbar, tmp_button,
+			    _("Go to the Right Page"),
+			    "go_right_page");
+  paned_notebook->right_arrow = tmp_button;
+  gtk_widget_set_sensitive(tmp_button, FALSE);
+
   gtk_toolbar_append_widget(paned_notebook->toolbar,
 			    GTK_WIDGET(paned_notebook->contents_menu),
-			    _("Select the page to be shown"),
+			    _("Select the Page to be Shown"),
 			    "item_selection_menu");
+  gtk_widget_set_sensitive(GTK_WIDGET(paned_notebook->contents_menu), FALSE);
 
   paned_notebook->item_contents = MOD_NOTEBOOK(mod_notebook_new());
   mod_notebook_set_scrollable(paned_notebook->item_contents, TRUE);
@@ -896,9 +929,6 @@ paned_notebook_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 	  gtk_widget_size_allocate(paned->child1, &child_allocation);
 	  if (paned_notebook->position_zero_hack)
 	    {
-#if 0
-	      fprintf(stderr, "Checkpoint #1\n");
-#endif
 	      gtk_widget_unmap(paned->child1);
 	      gtk_widget_set_child_visible(paned->child1, TRUE);
 	      gtk_widget_map(paned->child1);
@@ -1187,6 +1217,9 @@ paned_notebook_open_item_view(PanedNotebook *paned_notebook, gpointer item,
 		     GTK_WIDGET(paned_notebook->contents_pane));
     }
 
+  if (paned_notebook->number_of_items_shown == 0)
+    gtk_widget_set_sensitive(GTK_WIDGET(paned_notebook->contents_menu), TRUE);
+
   if (paned_notebook->number_of_items_shown++ == 0 || in_tab)
     {
       mod_notebook_append_page(paned_notebook->item_contents, item_view,
@@ -1206,6 +1239,11 @@ paned_notebook_open_item_view(PanedNotebook *paned_notebook, gpointer item,
 	  mod_notebook_set_current_page(paned_notebook->item_contents, page);
 	}
 
+      page = mod_notebook_get_current_page(paned_notebook->item_contents);
+      gtk_widget_set_sensitive(paned_notebook->left_arrow, page > 0);
+      gtk_widget_set_sensitive(paned_notebook->right_arrow,
+			       (page
+				< paned_notebook->number_of_items_shown - 1));
       return;
     }
 
@@ -1277,7 +1315,7 @@ close_nth_page(PanedNotebook *paned_notebook, int page)
   g_return_if_fail(widget != NULL);
 
   relation = find_item_view_relation(paned_notebook, widget);
-#if 1 || DEBUG_WIDGET
+#if DEBUG_WIDGET
   g_return_if_fail(relation != NULL && relation->item_view == widget);
 #else
   if (relation == NULL || relation->item_view != widget)
@@ -1308,14 +1346,18 @@ close_nth_page(PanedNotebook *paned_notebook, int page)
 
   paned_notebook->number_of_items_shown--;
 
-  if (paned_notebook->number_of_items_shown == 0
-      && !paned_notebook->always_show_contents_pane)
+  if (paned_notebook->number_of_items_shown == 0)
     {
-      if (gtk_paned_get_position(GTK_PANED(paned_notebook)) == 0)
-	paned_notebook->position_zero_hack = TRUE;
-
-      gtk_container_remove(GTK_CONTAINER(paned_notebook),
-			   GTK_WIDGET(paned_notebook->contents_pane));
+      gtk_widget_set_sensitive(GTK_WIDGET(paned_notebook->contents_menu),
+			       FALSE);
+      if (!paned_notebook->always_show_contents_pane)
+	{
+	  if (gtk_paned_get_position(GTK_PANED(paned_notebook)) == 0)
+	    paned_notebook->position_zero_hack = TRUE;
+	  
+	  gtk_container_remove(GTK_CONTAINER(paned_notebook),
+			       GTK_WIDGET(paned_notebook->contents_pane));
+	}
     }
 
   if (paned_notebook->number_of_items_shown == 1
@@ -1334,6 +1376,11 @@ close_nth_page(PanedNotebook *paned_notebook, int page)
 
   OCHU_OBJECT_UNREF(G_OBJECT(widget));
 
+  page = mod_notebook_get_current_page(paned_notebook->item_contents);
+  gtk_widget_set_sensitive(paned_notebook->left_arrow, page > 0);
+  gtk_widget_set_sensitive(paned_notebook->right_arrow,
+			   page < paned_notebook->number_of_items_shown - 1);
+
   gtk_container_remove(GTK_CONTAINER(paned_notebook->contents_list),
 		       menu_item);
   gtk_option_menu_set_history(paned_notebook->contents_menu, 0);
@@ -1348,6 +1395,11 @@ switch_page_cb(ModNotebook *notebook, ModNotebookPage *page, guint page_num,
   GtkWidget *new_current_page = mod_notebook_get_nth_page(notebook, page_num);
   GtkWidget *previous_page = paned_notebook->current_item_view;
 
+  gtk_widget_set_sensitive(paned_notebook->left_arrow, page_num > 0);
+  gtk_widget_set_sensitive(paned_notebook->right_arrow,
+			   (page_num
+			    < paned_notebook->number_of_items_shown - 1));
+
   if (previous_page == new_current_page)
     return;
 
@@ -1453,18 +1505,7 @@ close_item_view_button_cb(gpointer unused, PanedNotebook *paned_notebook)
 
 
 static void
-go_forward_button_cb(gpointer unused, PanedNotebook *paned_notebook)
-{
-  MAIN_THREAD_EVENT_CALLBACK();
-
-  mod_notebook_next_page(paned_notebook->item_contents);
-
-  ENTER_MAIN_EVENT_LOOP();
-}
-
-
-static void
-go_back_button_cb(gpointer unused, PanedNotebook *paned_notebook)
+go_left_button_cb(gpointer unused, PanedNotebook *paned_notebook)
 {
   MAIN_THREAD_EVENT_CALLBACK();
 
@@ -1472,3 +1513,14 @@ go_back_button_cb(gpointer unused, PanedNotebook *paned_notebook)
 
   ENTER_MAIN_EVENT_LOOP();
 }
+
+
+static void
+go_right_button_cb(gpointer unused, PanedNotebook *paned_notebook)
+{
+  MAIN_THREAD_EVENT_CALLBACK();
+
+  mod_notebook_next_page(paned_notebook->item_contents);
+
+  ENTER_MAIN_EVENT_LOOP();
+}
