@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: paned_notebook.h,v 1.4 2003/12/30 21:53:28 fuyu Exp $
+ * $Id: paned_notebook.h,v 1.5 2004/01/10 18:26:18 fuyu Exp $
  */
 
 #ifndef _PANED_NOTEBOOK_H_
@@ -90,6 +90,8 @@ struct _PanedNotebook
 
 
   /* tool_bar */
+  GtkWidget *left_arrow;
+  GtkWidget *right_arrow;
   GtkOptionMenu *contents_menu;
 
   /* contents_menu */
