@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: threadlist_filter.c,v 1.9 2003/12/30 21:53:28 fuyu Exp $
+ * $Id: threadlist_filter.c,v 1.12 2004/01/11 03:03:58 fuyu Exp $
  */
 
 #include "config.h"
@@ -110,8 +110,8 @@ static int threadlist_filter_rule_default_ignore_threshold;
 
 static const gchar *show_all;
 static const gchar *hide_all;
-static const gchar *show_favorite;
-static const gchar *hide_non_favorite;
+static const gchar *show_marked;
+static const gchar *hide_unmarked;
 static const gchar *show_seen;
 static const gchar *hide_unseen;
 static const gchar *show_seen_offline;
@@ -203,23 +203,23 @@ threadlist_filter_setting_class_init(ThreadlistFilterSettingClass *klass)
   klass->restore_default = threadlist_filter_setting_restore_default;
   klass->changed = threadlist_filter_setting_changed;
 
-  show_all = _("Show all threads");
-  hide_all = _("Hide all threads");
-  show_favorite = _("Show favorite threads");
-  hide_non_favorite = _("Hide non favorite threads");
-  show_seen = _("Show threads ever seen");
-  hide_unseen = _("Hide threads never seen");
-  show_seen_offline = _("Show threads ever seen during offline");
-  hide_unseen_offline	= _("Hide threads never seen during offline");
-  show_new_comer = _("Show new comer threads");
-  show_new_res = _("Show threads having new responses");
-  hide_without_new_res = _("Hide threads having no unread responses");
-  show_dat_dropped = _("Show threads whose DAT was dropped");
-  hide_ignored = _("Hide threads too much times ignored");
-  hide_hidden = _("Hide hidden threads");
-  show_good_title = _("Show threads having good title");
-  hide_no_good_title = _("Hide threads having no good title");
-  invalid_rule = _("Invalid rule");
+  show_all = _("Show All Threads");
+  hide_all = _("Hide All Threads");
+  show_marked = _("Show Marked Threads");
+  hide_unmarked = _("Hide Unmarked Threads");
+  show_seen = _("Show Threads Ever Seen");
+  hide_unseen = _("Hide Threads Never Seen");
+  show_seen_offline = _("Show Threads Ever Seen during Offline");
+  hide_unseen_offline	= _("Hide Threads Never Seen during Offline");
+  show_new_comer = _("Show New Comer Threads");
+  show_new_res = _("Show Threads having New Responses");
+  hide_without_new_res = _("Hide Threads having NO Unread Responses");
+  show_dat_dropped = _("Show Threads whose DAT was Dropped");
+  hide_ignored = _("Hide Threads Too Much Times Ignored");
+  hide_hidden = _("Hide Hidden Threads");
+  show_good_title = _("Show Threads having Good Title");
+  hide_no_good_title = _("Hide Threads having NO Good Title");
+  invalid_rule = _("Invalid Rule");
 }
 
 
@@ -917,19 +917,19 @@ setup_models(ThreadlistFilterSetting *setting)
 					   G_TYPE_INT, G_TYPE_INT);
   setting->all_rules_model = GTK_TREE_MODEL(store);
 
-  gtk_list_store_append(store, &setting->show_favorite.all_rules_iter);
-  gtk_list_store_set(store, &setting->show_favorite.all_rules_iter,
-	FILTER_RULE, THREADLIST_FILTER_RULE_SHOW_FAVORITE,
+  gtk_list_store_append(store, &setting->show_marked.all_rules_iter);
+  gtk_list_store_set(store, &setting->show_marked.all_rules_iter,
+	FILTER_RULE, THREADLIST_FILTER_RULE_SHOW_MARKED,
 	FILTER_RULE_PRIORITY, -1,
 	-1);
-  setting->show_favorite.priority = -1;
+  setting->show_marked.priority = -1;
 
-  gtk_list_store_append(store, &setting->hide_non_favorite.all_rules_iter);
-  gtk_list_store_set(store, &setting->hide_non_favorite.all_rules_iter,
-	FILTER_RULE, THREADLIST_FILTER_RULE_HIDE_NON_FAVORITE,
+  gtk_list_store_append(store, &setting->hide_unmarked.all_rules_iter);
+  gtk_list_store_set(store, &setting->hide_unmarked.all_rules_iter,
+	FILTER_RULE, THREADLIST_FILTER_RULE_HIDE_UNMARKED,
 	FILTER_RULE_PRIORITY, -1,
 	-1);
-  setting->hide_non_favorite.priority = -1;
+  setting->hide_unmarked.priority = -1;
 
   gtk_list_store_append(store, &setting->show_seen.all_rules_iter);
   gtk_list_store_set(store, &setting->show_seen.all_rules_iter,
@@ -1240,11 +1240,11 @@ threadlist_filter_setting_init(ThreadlistFilterSetting *setting)
 
   tmp_container = gtk_hbox_new(FALSE, 0);
 
-  tmp_widget = gtk_label_new(_("Threads not filtered:"));
+  tmp_widget = gtk_label_new(_("Threads Not Filtered:"));
   gtk_widget_show(tmp_widget);
   gtk_box_pack_start(GTK_BOX(tmp_container), tmp_widget, FALSE, FALSE, 0);
 
-  tmp_widget = gtk_radio_button_new_with_label(NULL, _("Show all"));
+  tmp_widget = gtk_radio_button_new_with_label(NULL, _("Show All"));
   g_signal_connect(G_OBJECT(tmp_widget), "toggled",
 		   G_CALLBACK(radio_button_toggled_cb), setting);
 
@@ -1254,7 +1254,7 @@ threadlist_filter_setting_init(ThreadlistFilterSetting *setting)
 
   tmp_widget
     = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(tmp_widget),
-						  _("Hide all"));
+						  _("Hide All"));
   gtk_widget_show(tmp_widget);
   gtk_box_pack_start(GTK_BOX(tmp_container), tmp_widget, FALSE, FALSE, 0);
   setting->default_hide_all_button = GTK_TOGGLE_BUTTON(tmp_widget);
@@ -1286,7 +1286,7 @@ threadlist_filter_setting_init(ThreadlistFilterSetting *setting)
 
   tmp_container = gtk_vbutton_box_new();
 
-  tmp_widget = gtk_button_new_with_label(_("Restore the default settings"));
+  tmp_widget = gtk_button_new_with_label(_("Restore the Default Settings"));
   setting->restore_default_button = tmp_widget;
   g_signal_connect(G_OBJECT(tmp_widget), "clicked",
 		   G_CALLBACK(restore_default_button_cb), setting);
@@ -1345,11 +1345,11 @@ get_threadlist_filter_rule_string(int rule)
     case THREADLIST_FILTER_RULE_HIDE_ALL:
       return hide_all;
 
-    case THREADLIST_FILTER_RULE_SHOW_FAVORITE:
-      return show_favorite;
+    case THREADLIST_FILTER_RULE_SHOW_MARKED:
+      return show_marked;
 
-    case THREADLIST_FILTER_RULE_HIDE_NON_FAVORITE:
-      return hide_non_favorite;
+    case THREADLIST_FILTER_RULE_HIDE_UNMARKED:
+      return hide_unmarked;
       
     case THREADLIST_FILTER_RULE_SHOW_THREAD_SEEN:
       return show_seen;
@@ -1418,11 +1418,11 @@ get_relation_for_rule(ThreadlistFilterSetting *setting, int rule)
 {
   switch (rule)
     {
-    case THREADLIST_FILTER_RULE_SHOW_FAVORITE:
-      return &setting->show_favorite;
+    case THREADLIST_FILTER_RULE_SHOW_MARKED:
+      return &setting->show_marked;
       
-    case THREADLIST_FILTER_RULE_HIDE_NON_FAVORITE:
-      return &setting->hide_non_favorite;
+    case THREADLIST_FILTER_RULE_HIDE_UNMARKED:
+      return &setting->hide_unmarked;
       
     case THREADLIST_FILTER_RULE_SHOW_THREAD_SEEN:
       return &setting->show_seen;
@@ -1617,35 +1617,35 @@ threadlist_filter_check_thread(ThreadlistFilter *filter,
 	case THREADLIST_FILTER_RULE_HIDE_ALL:
 	  return FALSE;
 
-	case THREADLIST_FILTER_RULE_SHOW_FAVORITE:
-	  if (thread_info->view_flags & BBS_THREAD_FAVORITE)
+	case THREADLIST_FILTER_RULE_SHOW_MARKED:
+	  if (thread_info->view_flags & BBS_THREAD_MARKED)
 	    return TRUE;
 	  break;
       
-	case THREADLIST_FILTER_RULE_HIDE_NON_FAVORITE:
-	  if ((thread_info->view_flags & BBS_THREAD_FAVORITE) == 0)
+	case THREADLIST_FILTER_RULE_HIDE_UNMARKED:
+	  if ((thread_info->view_flags & BBS_THREAD_MARKED) == 0)
 	    return FALSE;
 	  break;
       
 	case THREADLIST_FILTER_RULE_SHOW_THREAD_SEEN:
-	  if (thread->number_of_responses_read > 0)
+	  if (ochusha_bbs_thread_get_number_of_responses_read(thread) > 0)
 	    return TRUE;
 	  break;
 
 	case THREADLIST_FILTER_RULE_HIDE_THREAD_NEVER_SEEN:
-	  if (thread->number_of_responses_read == 0)
+	  if (ochusha_bbs_thread_get_number_of_responses_read(thread) == 0)
 	    return FALSE;
 	  break;
 
 	case THREADLIST_FILTER_RULE_SHOW_THREAD_SEEN_OFFLINE:
 	  if (application->config.offline
-	      && thread->number_of_responses_read > 0)
+	      && ochusha_bbs_thread_get_number_of_responses_read(thread) > 0)
 	    return TRUE;
 	  break;
 
 	case THREADLIST_FILTER_RULE_HIDE_THREAD_NEVER_SEEN_OFFLINE:
 	  if (application->config.offline
-	      && thread->number_of_responses_read == 0)
+	      && ochusha_bbs_thread_get_number_of_responses_read(thread) == 0)
 	    return FALSE;
 	  break;
 
@@ -1656,19 +1656,20 @@ threadlist_filter_check_thread(ThreadlistFilter *filter,
 	  break;
 
 	case THREADLIST_FILTER_RULE_SHOW_THREAD_WITH_NEW_RESPONSES:
-	  if (thread->number_of_responses_on_server
-	      > thread->number_of_responses_read)
+	  if (ochusha_bbs_thread_get_number_of_responses_on_server(thread)
+	      > ochusha_bbs_thread_get_number_of_responses_read(thread))
 	    return TRUE;
 	  break;
 
 	case THREADLIST_FILTER_RULE_HIDE_THREAD_WITHOUT_NEW_RESPONSES:
-	  if (thread->number_of_responses_on_server
-	      <= thread->number_of_responses_read)
+	  if (ochusha_bbs_thread_get_number_of_responses_on_server(thread)
+	      <= ochusha_bbs_thread_get_number_of_responses_read(thread))
 	    return FALSE;
 	  break;
 
 	case THREADLIST_FILTER_RULE_SHOW_DAT_DROPPED_THREAD:
-	  if (thread->flags & OCHUSHA_BBS_THREAD_DAT_DROPPED)
+	  if (ochusha_bbs_thread_get_flags(thread)
+	      & OCHUSHA_BBS_THREAD_DAT_DROPPED)
 	    return TRUE;
 	  break;
 
