@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: virtual_board.c,v 1.8 2004/01/10 17:23:38 fuyu Exp $
+ * $Id: virtual_board.c,v 1.9 2004/01/13 01:23:08 fuyu Exp $
  */
 
 #include "config.h"
@@ -213,6 +213,100 @@ virtual_board_remove_thread(VirtualBoard *virtual_board,
 }
 
 
+static int
+compare_by_id(OchushaBBSThread *a, OchushaBBSThread *b)
+{
+  OchushaBBSThread *real_a;
+  OchushaBBSThread *real_b;
+  char tmp_a[PATH_MAX];
+  char tmp_b[PATH_MAX];
+  int id_a;
+  int id_b;
+  int result;
+
+  g_return_val_if_fail(IS_THREAD_PROXY(a) && IS_THREAD_PROXY(b), 0);
+  real_a = THREAD_PROXY(a)->real_thread;
+  real_b = THREAD_PROXY(b)->real_thread;
+
+  if (real_a == NULL || real_b == NULL)
+    return strcmp(ochusha_bbs_thread_get_url(a),
+		  ochusha_bbs_thread_get_url(b));
+
+  g_strlcpy(tmp_a, ochusha_bulletin_board_get_server(real_a->board), PATH_MAX);
+  g_strlcpy(tmp_b, ochusha_bulletin_board_get_server(real_b->board), PATH_MAX);
+  g_strreverse(tmp_a);
+  g_strreverse(tmp_b);
+  result = strcmp(tmp_a, tmp_b);
+  if (result != 0)
+    return result;
+
+  snprintf(tmp_a, PATH_MAX, "%s%s",
+	   real_a->board->base_path, real_a->board->id);
+  snprintf(tmp_b, PATH_MAX, "%s%s",
+	   real_b->board->base_path, real_b->board->id);
+  result = strcmp(tmp_a, tmp_b);
+  if (result != 0)
+    return result;
+
+  if (real_a->id != NULL && sscanf(real_a->id, "%d", &id_a) == 1
+      && real_b->id != NULL && sscanf(real_b->id, "%d", &id_b) == 1)
+    return id_a - id_b;
+
+  return strcmp(ochusha_bbs_thread_get_url(a),
+		ochusha_bbs_thread_get_url(b));
+}
+
+
+void
+collect_unread_thread(OchushaBBSThread *thread, GSList **removed_list_p)
+{
+  if (ochusha_bbs_thread_get_number_of_responses_read(thread) == 0)
+    *removed_list_p = g_slist_append(*removed_list_p, thread);
+}
+
+
+void
+remove_thread(OchushaBBSThread *thread, VirtualBoard *virtual_board)
+{
+  virtual_board_remove_thread(virtual_board, thread);
+}
+
+
+void
+virtual_board_clean_up_threadlist(VirtualBoard *virtual_board,
+				  gboolean remove_unread_thread)
+{
+  OchushaBulletinBoard *board;
+  BulletinBoardGUIInfo *info;
+  g_return_if_fail(IS_VIRTUAL_BOARD(virtual_board));
+
+  board = OCHUSHA_BULLETIN_BOARD(virtual_board);
+
+  if (remove_unread_thread)
+    {
+      GSList *removed_list = NULL;
+      g_slist_foreach(board->thread_list, (GFunc)collect_unread_thread,
+		      &removed_list);
+      g_slist_foreach(removed_list, (GFunc)remove_thread, virtual_board);
+      g_slist_free(removed_list);
+    }
+
+  board->thread_list = g_slist_sort(board->thread_list,
+				    (GCompareFunc)compare_by_id);
+  info = ensure_bulletin_board_info(board, virtual_board->application);
+  if (info->use_static_threadlist)
+    {
+      g_slist_free(virtual_board->recent_thread_list);
+      virtual_board->recent_thread_list = g_slist_copy(board->thread_list);
+    }
+  else
+    {
+      g_slist_free(virtual_board->recent_thread_list);
+      virtual_board->recent_thread_list = NULL;
+    }
+}
+
+
 static char *
 virtual_board_generate_base_path(OchushaBulletinBoard *board, const char *url)
 {
@@ -246,6 +340,7 @@ virtual_board_bbs_thread_new(OchushaBulletinBoard *board, const char *id,
   VirtualBoard *virtual_board;
   OchushaBBSThread *thread;
   OchushaBulletinBoard *real_board = NULL;
+  BulletinBoardGUIInfo *info;
   g_return_val_if_fail(IS_VIRTUAL_BOARD(board), NULL);
   g_return_val_if_fail(id != NULL && title != NULL, NULL);
 
@@ -264,6 +359,7 @@ virtual_board_bbs_thread_new(OchushaBulletinBoard *board, const char *id,
     {
       if (real_board != NULL)
 	{
+	  OchushaBBSThread *real_thread;
 	  if (g_slist_find(virtual_board->board_list, real_board) == NULL)
 	    {
 	      virtual_board->board_list
@@ -271,11 +367,27 @@ virtual_board_bbs_thread_new(OchushaBulletinBoard *board, const char *id,
 #if 0
 	      fprintf(stderr, "board added: %s\n", real_board->base_url);
 #endif
+	      if (real_board->thread_list == NULL)
+		ochusha_bulletin_board_read_threadlist_xml(real_board,
+							   &virtual_board->application->config,
+							   virtual_board->application->session_subdir,
+							   TRUE);
 	    }
+	  real_thread = ochusha_bulletin_board_lookup_bbs_thread_by_url(real_board, id);
+	  if (real_thread != NULL)
+	    thread_proxy_set_real_thread(THREAD_PROXY(thread), real_thread);
+	  else
+	    fprintf(stderr, "Couldn't find real_thread by URL: %s\n", id);
 	}
     }
 
-  if (virtual_board->recent_thread_list != NULL)
+  info = ensure_bulletin_board_info(board, virtual_board->application);
+  if (info->use_static_threadlist)
+    {
+      virtual_board->recent_thread_list
+	= g_slist_append(virtual_board->recent_thread_list, thread);
+    }
+  else if (virtual_board->recent_thread_list != NULL)
     {
       g_slist_free(virtual_board->recent_thread_list);
       virtual_board->recent_thread_list = NULL;
@@ -390,6 +502,7 @@ virtual_board_refresh_threadlist(OchushaBulletinBoard *board,
       each_thread_cb, callback_data,
       NULL,
     };
+  BulletinBoardGUIInfo *info;
 
   g_return_val_if_fail(IS_VIRTUAL_BOARD(board), FALSE);
 
@@ -397,7 +510,10 @@ virtual_board_refresh_threadlist(OchushaBulletinBoard *board,
 
   args.application = virtual_board->application;
 
-  if (args.mode == OCHUSHA_NETWORK_BROKER_CACHE_ONLY
+  info = ensure_bulletin_board_info(board, virtual_board->application);
+
+  if ((args.mode == OCHUSHA_NETWORK_BROKER_CACHE_ONLY
+       || info->use_static_threadlist)
       && virtual_board->recent_thread_list != NULL)
     g_slist_foreach(virtual_board->recent_thread_list,
 		    (GFunc)each_thread_cb, callback_data);
