@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_async_buffer.c,v 1.18 2003/12/30 21:53:17 fuyu Exp $
+ * $Id: ochusha_async_buffer.c,v 1.21 2004/01/09 22:05:26 fuyu Exp $
  */
 
 #include "config.h"
@@ -563,6 +563,16 @@ ochusha_async_buffer_new_with_file_mmap(int fd)
 }
 
 
+gboolean
+ochusha_async_buffer_reset(OchushaAsyncBuffer *buffer)
+{
+  g_return_val_if_fail(OCHUSHA_IS_ASYNC_BUFFER(buffer), FALSE);
+  /* g_return_val_if_fail(buffer->number_of_active_users == 0, FALSE); */
+  buffer->fixed = FALSE;
+  return TRUE;
+}
+
+
 /* ƱߴϢ */
 /*
  * MEMO: OchushaAsyncBufferȤäƤƤΥåɤ⤷Υå
@@ -663,6 +673,14 @@ ochusha_async_buffer_resume_all(void)
 }
 
 
+void
+ochusha_async_buffer_terminate_all(void)
+{
+  g_slist_foreach(ochusha_async_buffer_list,
+		  (GFunc)ochusha_async_buffer_terminate, NULL);
+}
+
+
 void
 ochusha_async_buffer_suspend(OchushaAsyncBuffer *buffer)
 {
@@ -925,6 +943,9 @@ ochusha_async_buffer_append_data(OchushaAsyncBuffer *buffer, const char *data,
 }
 
 
+#define MAXIMUM_CHUNK_SIZE	65536
+
+
 gboolean
 ochusha_async_buffer_read_file(OchushaAsyncBuffer *buffer, int fd)
 {
@@ -950,7 +971,8 @@ ochusha_async_buffer_read_file(OchushaAsyncBuffer *buffer, int fd)
 	}
 
       buf = (char *)buffer->buffer + buffer->length;
-      chunk_size = buffer->buffer_length - buffer->length;
+      chunk_size = MIN(buffer->buffer_length - buffer->length,
+		       MAXIMUM_CHUNK_SIZE);
       result = gzread(gzfile, buf, chunk_size);
       if (result == -1)
 	{
