@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_network_broker.c,v 1.41 2004/01/05 12:17:04 fuyu Exp $
+ * $Id: ochusha_network_broker.c,v 1.43 2004/01/09 21:17:50 fuyu Exp $
  */
 
 #include "config.h"
@@ -437,6 +437,7 @@ typedef struct _NetworkBrokerJobArgs
 {
   OchushaNetworkBroker *broker;
 
+  int length;
   char *url;
   char *if_modified_since;
 
@@ -548,6 +549,10 @@ worker_sync_object_finalize(GObject *object)
 {
   WorkerSyncObject *sync_object = OCHUSHA_WORKER_SYNC_OBJECT(object);
 
+#if 0
+  fprintf(stderr, "worker_sync_object_finalize.\n");
+#endif
+
   if (sync_object->lock_and_cond_initialized)
     {
       if (pthread_mutex_destroy(&sync_object->lock) != 0)
@@ -623,6 +628,11 @@ timeout_cb(WorkerSyncObject *sync_object)
   if (!finished)
     return TRUE;
 
+#if DEBUG_NETWORK_MOST
+  fprintf(stderr, "finishing: timeout_cb(%p) count=%d\n",
+	  sync_object, G_OBJECT(sync_object)->ref_count);
+#endif
+
   OCHU_OBJECT_UNREF(G_OBJECT(sync_object));
 
   return FALSE;
@@ -651,7 +661,8 @@ poll_cb(GIOChannel *channel, GIOCondition condition,
   UNLOCK_WORKER_SYNC_OBJECT(sync_object);
 
 #if DEBUG_NETWORK_MOST
-  fprintf(stderr, "leaving: poll_cb(%p)\n", sync_object);
+  fprintf(stderr, "leaving: poll_cb(%p): count=%d\n",
+	  sync_object, G_OBJECT(sync_object)->ref_count);
 #endif
   OCHU_OBJECT_UNREF(G_OBJECT(sync_object));
 
@@ -663,16 +674,21 @@ static void
 register_polling_function_for_read(WorkerSyncObject *sync_object, int fd)
 {
   LOCK_WORKER_SYNC_OBJECT(sync_object);
-  OCHU_OBJECT_REF(G_OBJECT(sync_object));
   if (sync_object->poll_id == 0)
     {
       GIOChannel *channel = g_io_channel_unix_new(fd);
+      OCHU_OBJECT_REF(G_OBJECT(sync_object));
       sync_object->poll_id
 	= g_io_add_watch_full(channel,
 			      G_PRIORITY_DEFAULT,
 			      G_IO_IN|G_IO_PRI|G_IO_ERR|G_IO_HUP|G_IO_NVAL,
 			      (GIOFunc)poll_cb, sync_object, NULL);
       g_io_channel_unref(channel);
+#if DEBUG_NETWORK_MOST
+      fprintf(stderr,
+	      "leaving: register_polling_function_for_read(%p): count=%d\n",
+	      sync_object, G_OBJECT(sync_object)->ref_count);
+#endif
     }
   sync_object->read_ok = FALSE;
   UNLOCK_WORKER_SYNC_OBJECT(sync_object);
@@ -705,6 +721,9 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
   const char *tmp_pos;
   gboolean force_gunzip;	/* Ķad-hock塼fordat.gz */
 
+  ochusha_async_buffer_reset(buffer);
+  ochusha_async_buffer_update_length(buffer, 0);
+
   if ((tmp_pos = strstr(url, ".dat.gz")) != NULL && tmp_pos[7] == '\0')
     force_gunzip = TRUE;
   else
@@ -1238,7 +1257,8 @@ ochusha_network_broker_worker_sync_object_free(WorkerSyncObject *sync_object)
   g_return_if_fail(OCHUSHA_IS_WORKER_SYNC_OBJECT(sync_object));
 
 #if DEBUG_NETWORK_MOST
-  fprintf(stderr, "entering: sync_object_free(%p)\n", sync_object);
+  fprintf(stderr, "entering: sync_object_free(%p): count=%d\n",
+	  sync_object, G_OBJECT(sync_object)->ref_count);
 #endif
 
   OCHU_OBJECT_UNREF(G_OBJECT(sync_object));
@@ -1247,6 +1267,7 @@ ochusha_network_broker_worker_sync_object_free(WorkerSyncObject *sync_object)
 
 static OchushaAsyncBuffer *
 ochusha_network_broker_employ_worker_thread(OchushaNetworkBroker *broker,
+					    OchushaAsyncBuffer *buffer,
 					    const char *url,
 					    const char *if_modified_since,
 					    JobFunc *job_function)
@@ -1256,15 +1277,16 @@ ochusha_network_broker_employ_worker_thread(OchushaNetworkBroker *broker,
   WorkerSyncObject *sync_object = worker_sync_object_new();
   OchushaNetworkBrokerBufferStatus *status
     = G_NEW0(OchushaNetworkBrokerBufferStatus, 1);
-  OchushaAsyncBuffer *buffer = ochusha_async_buffer_new(NULL, 0, NULL);
+  if (buffer == NULL || !ochusha_async_buffer_reset(buffer))
+    buffer = ochusha_async_buffer_new(NULL, 0, NULL);
 
   g_object_set_qdata_full(G_OBJECT(buffer), worker_sync_object_id, sync_object,
 		(GDestroyNotify)ochusha_network_broker_worker_sync_object_free);
   g_object_set_qdata_full(G_OBJECT(buffer), broker_buffer_status_id, status,
 		(GDestroyNotify)ochusha_network_broker_buffer_status_free);
 
-  g_signal_connect(G_OBJECT(buffer), "wakeup_now",
-		   G_CALLBACK(wakeup_now_cb), sync_object);
+  g_signal_connect_object(G_OBJECT(buffer), "wakeup_now",
+			  G_CALLBACK(wakeup_now_cb), sync_object, 0);
 
   sync_object->timeout_id = g_timeout_add_full(G_PRIORITY_DEFAULT - 1,
 					       POLLING_INTERVAL_MILLIS,
@@ -1274,10 +1296,13 @@ ochusha_network_broker_employ_worker_thread(OchushaNetworkBroker *broker,
   status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_UNKNOWN;
 
   args = G_NEW0(NetworkBrokerJobArgs, 1);
+  args->length = buffer->length;
+  buffer->length = 0;
   args->broker = broker;
   args->url = G_STRDUP(url);
   if (if_modified_since != NULL
-      && ochusha_config_cache_file_exist(broker->config, url))
+      && (args->length > 0
+	  || ochusha_config_cache_file_exist(broker->config, url)))
     args->if_modified_since = G_STRDUP(if_modified_since);
 
   g_object_set_qdata_full(G_OBJECT(buffer), broker_job_args_id,
@@ -1350,7 +1375,43 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
       return;
     }
 
-  len = read_cache_to_buffer(broker->config, url, buffer);
+  if (args->length == 0)
+    len = read_cache_to_buffer(broker->config, url, buffer);
+  else
+    {
+      len = args->length;
+      if (!ochusha_async_buffer_update_length(buffer, len))
+	{
+	  g_signal_emit(G_OBJECT(broker),
+			broker_signals[ACCESS_TERMINATED_SIGNAL],
+			0,
+			buffer,
+			&signal_result);
+
+	  status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
+	  g_signal_emit(G_OBJECT(broker),
+			broker_signals[ACCESS_FAILED_SIGNAL],
+			0,
+			buffer,
+			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
+			_("Access terminated."),
+			&signal_result);
+
+	  ochusha_async_buffer_fix(buffer);
+	  g_object_set_qdata(G_OBJECT(buffer), broker_job_args_id, NULL);
+	  OCHU_OBJECT_UNREF(G_OBJECT(buffer));
+
+	  snprintf(message, LOG_MESSAGE_SIZE,
+		   _("Access terminated: %s\n"), url);
+	  ochusha_network_broker_output_log(broker, message);
+
+	  LOCK_WORKER_SYNC_OBJECT(sync_object);
+	  sync_object->finished = TRUE;
+	  UNLOCK_WORKER_SYNC_OBJECT(sync_object);
+
+	  return;
+	}
+    }
 
   if (len > 0)
     {	/* å夢ξ */
@@ -1916,8 +1977,6 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   fprintf(stderr, "refresh_cache_after_read() thread is invoked.\n");
 #endif
 
-  
-
   if (!ochusha_async_buffer_active_ref(buffer))
     {
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
@@ -1941,6 +2000,8 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   else
     {
       /* ̿Ԥå夬(if_modified_sinceNULL)ʾ */
+      ochusha_async_buffer_reset(buffer);
+      ochusha_async_buffer_update_length(buffer, 0);
       if (read_cache_to_buffer(broker->config, args->url, buffer))
 	{
 	  status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_HIT;
@@ -2154,6 +2215,7 @@ background_read_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
  */
 OchushaAsyncBuffer *
 ochusha_network_broker_read_from_url(OchushaNetworkBroker *broker,
+				     OchushaAsyncBuffer *buffer,
 				     const char *url,
 				     const char *if_modified_since,
 				     OchushaNetworkBrokerCacheMode mode)
@@ -2171,7 +2233,7 @@ ochusha_network_broker_read_from_url(OchushaNetworkBroker *broker,
       g_signal_emit(G_OBJECT(broker),
 		    broker_signals[ACCESS_FAILED_SIGNAL],
 		    0,
-		    NULL,
+		    buffer,
 		    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_NOT_CONNECTED,
 		    _("Network not connected on offline mode."),
 		    &signal_result);
@@ -2188,9 +2250,10 @@ ochusha_network_broker_read_from_url(OchushaNetworkBroker *broker,
       if (fd >= 0)
 	{
 	  WorkerJob *job = G_NEW0(WorkerJob, 1);
-	  OchushaAsyncBuffer *buffer = ochusha_async_buffer_new(NULL, 0, NULL);
 	  OchushaNetworkBrokerBufferStatus *status
 	    = G_NEW0(OchushaNetworkBrokerBufferStatus, 1);
+	  if (buffer == NULL || !ochusha_async_buffer_reset(buffer))
+	    buffer = ochusha_async_buffer_new(NULL, 0, NULL);
 
 	  g_object_set_qdata(G_OBJECT(buffer), filedesc_id, (gpointer)fd);
 	  g_object_set_qdata(G_OBJECT(buffer), broker_id, broker);
@@ -2218,7 +2281,7 @@ ochusha_network_broker_read_from_url(OchushaNetworkBroker *broker,
 	  g_signal_emit(G_OBJECT(broker),
 			broker_signals[ACCESS_FAILED_SIGNAL],
 			0,
-			NULL,
+			buffer,
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_CACHE_NOT_FOUND,
 			_("Couldn't find cache that should exist."),
 			&signal_result);
@@ -2230,7 +2293,7 @@ ochusha_network_broker_read_from_url(OchushaNetworkBroker *broker,
 	  g_signal_emit(G_OBJECT(broker),
 			broker_signals[ACCESS_FAILED_SIGNAL],
 			0,
-			NULL,
+			buffer,
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_NOT_CONNECTED,
 			_("Network not connected on offline mode."),
 			&signal_result);
@@ -2241,18 +2304,18 @@ ochusha_network_broker_read_from_url(OchushaNetworkBroker *broker,
   switch (mode)
     {
     case OCHUSHA_NETWORK_BROKER_CACHE_TRY_UPDATE:
-      return ochusha_network_broker_employ_worker_thread(broker,
+      return ochusha_network_broker_employ_worker_thread(broker, buffer,
 						url, if_modified_since,
 						(JobFunc *)try_update_cache);
 
     case OCHUSHA_NETWORK_BROKER_CACHE_TRY_REFRESH:
     case OCHUSHA_NETWORK_BROKER_CACHE_AS_IS:
-      return ochusha_network_broker_employ_worker_thread(broker,
+      return ochusha_network_broker_employ_worker_thread(broker, buffer,
 					url, if_modified_since,
 					(JobFunc *)refresh_cache_after_read);
 
     case OCHUSHA_NETWORK_BROKER_CACHE_IGNORE:
-      return ochusha_network_broker_employ_worker_thread(broker,
+      return ochusha_network_broker_employ_worker_thread(broker, buffer,
 						    url, NULL,
 						    (JobFunc *)force_read);
 
@@ -2570,6 +2633,7 @@ force_read_synchronously(WorkerThread *employee, OchushaAsyncBuffer *buffer)
  */
 OchushaAsyncBuffer *
 ochusha_network_broker_read_from_cgi(OchushaNetworkBroker *broker,
+				     OchushaAsyncBuffer *buffer,
 				     const char *url)
 {
   g_return_val_if_fail(OCHUSHA_IS_NETWORK_BROKER(broker)
@@ -2581,7 +2645,7 @@ ochusha_network_broker_read_from_cgi(OchushaNetworkBroker *broker,
   if (broker->config->offline)
     return NULL;
 
-  return ochusha_network_broker_employ_worker_thread(broker, url, NULL,
+  return ochusha_network_broker_employ_worker_thread(broker, buffer, url, NULL,
 					(JobFunc *)force_read_synchronously);
 }
 
