@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_thread_jbbs.c,v 1.21 2003/12/30 17:22:07 fuyu Exp $
+ * $Id: ochusha_thread_jbbs.c,v 1.23 2004/01/08 19:44:59 fuyu Exp $
  */
 
 #include "config.h"
@@ -70,6 +70,7 @@ static char *ochusha_thread_jbbs_get_base_path(OchushaThread2ch *thread_2ch);
 static OchushaAsyncBuffer *ochusha_thread_jbbs_get_responses_source(
 					OchushaBBSThread *thread,
 					OchushaNetworkBroker *broker,
+					OchushaAsyncBuffer *buffer,
 					OchushaNetworkBrokerCacheMode mode);
 static gboolean ochusha_thread_jbbs_parse_responses(OchushaBBSThread *thread,
 						    OchushaAsyncBuffer *buffer,
@@ -107,6 +108,7 @@ static const char *ochusha_thread_jbbs_get_url_to_post(OchushaBBSThread *thread)
 static OchushaAsyncBuffer *ochusha_thread_jbbs_fake_responses_source(
 					OchushaThread2ch *thread_2ch,
 					OchushaNetworkBroker *broker,
+					OchushaAsyncBuffer *buffer,
 					OchushaNetworkBrokerCacheMode mode);
 
 
@@ -149,6 +151,13 @@ ochusha_thread_jbbs_class_init(OchushaThreadJBBSClass *klass)
 
   parent_class = g_type_class_peek_parent(klass);
 
+  b_class->get_board = NULL;
+  b_class->get_number_of_responses_on_server = NULL;
+  b_class->get_number_of_responses_read = NULL;
+  b_class->set_number_of_responses_read = NULL;
+  b_class->get_flags = NULL;
+  b_class->set_flags = NULL;
+
   b_class->get_responses_source = ochusha_thread_jbbs_get_responses_source;
   b_class->parse_responses = ochusha_thread_jbbs_parse_responses;
   b_class->get_url
@@ -231,6 +240,7 @@ ochusha_thread_jbbs_get_base_path(OchushaThread2ch *thread_2ch)
 static OchushaAsyncBuffer *
 ochusha_thread_jbbs_get_responses_source(OchushaBBSThread *thread,
 					 OchushaNetworkBroker *broker,
+					 OchushaAsyncBuffer *buffer,
 					 OchushaNetworkBrokerCacheMode mode)
 {
   OchushaThread2ch *thread_2ch;
@@ -242,7 +252,8 @@ ochusha_thread_jbbs_get_responses_source(OchushaBBSThread *thread,
   if (thread->flags & OCHUSHA_BBS_THREAD_DAT_DROPPED)
     mode = OCHUSHA_NETWORK_BROKER_CACHE_ONLY;
 
-  return ochusha_thread_jbbs_fake_responses_source(thread_2ch, broker, mode);
+  return ochusha_thread_jbbs_fake_responses_source(thread_2ch, broker, buffer,
+						   mode);
 }
 
 
@@ -1551,6 +1562,7 @@ unref_html_buffer(OchushaAsyncBuffer *html_buffer)
 static OchushaAsyncBuffer *
 ochusha_thread_jbbs_fake_responses_source(OchushaThread2ch *thread_2ch,
 					  OchushaNetworkBroker *broker,
+					  OchushaAsyncBuffer *buffer,
 					  OchushaNetworkBrokerCacheMode mode)
 {
   OchushaAsyncBuffer *dat_buffer = NULL;
@@ -1578,40 +1590,49 @@ ochusha_thread_jbbs_fake_responses_source(OchushaThread2ch *thread_2ch,
 
   if (mode != OCHUSHA_NETWORK_BROKER_CACHE_IGNORE)
     {
-      int fd = ochusha_config_cache_open_file(broker->config, fake_dat_url,
-					      O_RDONLY);
-      if (fd >= 0)
+      if (buffer == NULL || !ochusha_async_buffer_reset(buffer))
 	{
-	  /* å夢ꡣ*/
-	  dat_buffer = ochusha_async_buffer_new(NULL, 0, NULL);
-	  incoming_offset = lseek(fd, 0, SEEK_END);
-	  lseek(fd, 0, SEEK_SET);
-	  if (ochusha_async_buffer_resize(dat_buffer, incoming_offset))
+	  int fd = ochusha_config_cache_open_file(broker->config, fake_dat_url,
+						  O_RDONLY);
+	  if (fd >= 0)
 	    {
-	      char *tmp_pos;
-	      char *tail_pos;
-	      incoming_offset = read(fd, (void *)dat_buffer->buffer,
-				     incoming_offset);
-	      if (!ochusha_async_buffer_update_length(dat_buffer,
-						      incoming_offset))
+	      /* å夢ꡣ*/
+	      dat_buffer = ochusha_async_buffer_new(NULL, 0, NULL);
+	      incoming_offset = lseek(fd, 0, SEEK_END);
+	      lseek(fd, 0, SEEK_SET);
+	      if (ochusha_async_buffer_resize(dat_buffer, incoming_offset))
 		{
+		  incoming_offset = read(fd, (void *)dat_buffer->buffer,
+					 incoming_offset);
+		  if (!ochusha_async_buffer_update_length(dat_buffer,
+							  incoming_offset))
+		    {
 #if DEBUG_ASYNC_BUFFER
-		  fprintf(stderr, "ochusha_utils_jbbs_fake_responses_source(): buffer may have been terminated.\n");
+		      fprintf(stderr, "ochusha_utils_jbbs_fake_responses_source(): buffer may have been terminated.\n");
 #endif
-		  /* ξϤɤ褦ʤ */
-		}
-
-	      /* ¸Υ쥹 */
-	      tmp_pos = (char *)dat_buffer->buffer;
-	      tail_pos = tmp_pos + incoming_offset;
-	      while ((tmp_pos = memchr(tmp_pos, '\n', tail_pos - tmp_pos))
-		     != NULL)
-		{
-		  number_of_cached_responses++;
-		  tmp_pos++;
+		      /* ξϤɤ褦ʤ */
+		    }
 		}
+	      close(fd);
+	    }
+	}
+      else
+	{
+	  dat_buffer = buffer;
+	  incoming_offset = buffer->length;
+	}
+
+      /* ¸Υ쥹 */
+      if (incoming_offset > 0)
+	{
+	  char *tmp_pos = (char *)dat_buffer->buffer;
+	  char *tail_pos = tmp_pos + incoming_offset;
+	  while ((tmp_pos = memchr(tmp_pos, '\n', tail_pos - tmp_pos))
+		 != NULL)
+	    {
+	      number_of_cached_responses++;
+	      tmp_pos++;
 	    }
-	  close(fd);
 	}
     }
 
@@ -1656,9 +1677,9 @@ ochusha_thread_jbbs_fake_responses_source(OchushaThread2ch *thread_2ch,
 
   if (cgi_url[0] != '\0')
 #if 1	/* ʤǤΤasync⡼ɤǥȼԤ롣*/
-    html_buffer = ochusha_network_broker_read_from_cgi(broker, cgi_url);
+    html_buffer = ochusha_network_broker_read_from_cgi(broker, NULL, cgi_url);
 #else
-    html_buffer = ochusha_network_broker_read_from_url(broker, cgi_url,
+    html_buffer = ochusha_network_broker_read_from_url(broker, NULL, cgi_url,
 						       NULL, mode);
 #endif
   else
@@ -1681,7 +1702,15 @@ ochusha_thread_jbbs_fake_responses_source(OchushaThread2ch *thread_2ch,
     }
 
   if (dat_buffer == NULL)
-    dat_buffer = ochusha_async_buffer_new(NULL, 0, NULL);
+    {
+      if (buffer == NULL || !ochusha_async_buffer_reset(buffer))
+	dat_buffer = ochusha_async_buffer_new(NULL, 0, NULL);
+      else
+	{
+	  dat_buffer = buffer;
+	  ochusha_async_buffer_update_length(buffer, 0);
+	}
+    }
 
   args = G_NEW0(HTML2DATConverterJobArgs, 1);
   args->html_buffer = html_buffer;
