@@ -9,9 +9,9 @@
 #define ONIGURUMA_H
 
 #define ONIGURUMA
-#define ONIGURUMA_VERSION_MAJOR   1
-#define ONIGURUMA_VERSION_MINOR   9
-#define ONIGURUMA_VERSION_TEENY   5
+#define ONIGURUMA_VERSION_MAJOR   2
+#define ONIGURUMA_VERSION_MINOR   0
+#define ONIGURUMA_VERSION_TEENY   0
 
 /* config parameters */
 #ifndef RE_NREGS
@@ -25,7 +25,7 @@
 #define REG_MAX_ERROR_MESSAGE_LEN            90
 
 #ifndef P_
-#ifdef __STDC__
+#if defined(__STDC__) || defined(_WIN32)
 # define P_(args) args
 #else
 # define P_(args) ()
@@ -55,41 +55,95 @@
 #endif
 
 #define REG_CHAR_TABLE_SIZE   256
+typedef unsigned char  UChar;
+typedef unsigned long  RegCodePoint;
+
+typedef struct {
+  RegCodePoint from;
+  RegCodePoint to;
+} RegCodePointRange;
+
+#define ENC_FOLD_MATCH_MAX_TARGET_NUM_SIZE  16
+typedef struct {
+  int    target_num;
+  int    target_byte_len[ENC_FOLD_MATCH_MAX_TARGET_NUM_SIZE];
+  UChar* target_str[ENC_FOLD_MATCH_MAX_TARGET_NUM_SIZE];
+} EncFoldMatchInfo;
 
-#define REGCODE_UNDEF         ((RegCharEncoding )0)
 
 #if defined(RUBY_PLATFORM) && defined(M17N_H)
+
 #define REG_RUBY_M17N
 typedef m17n_encoding*        RegCharEncoding;
-#define REGCODE_DEFAULT       REGCODE_UNDEF
+
 #else
-typedef const char*           RegCharEncoding;
+
+typedef struct {
+  const char  len_table[256];
+  const char* name;
+  int         max_enc_len;
+  int         is_fold_match;
+  int         ctype_support_level; /* sb-only/full */
+  int         is_continuous_sb_mb; /* code point is continuous from sb to mb */
+  RegCodePoint (*mbc_to_code)(UChar* p, UChar* end);
+  int    (*code_to_mbclen)(RegCodePoint code);
+  int    (*code_to_mbc_first)(RegCodePoint code);
+  int    (*code_to_mbc)(RegCodePoint code, UChar *buf);
+  int    (*mbc_to_lower)(UChar* p, UChar* lower);
+  int    (*mbc_is_case_ambig)(UChar* p);
+  int    (*code_is_ctype)(RegCodePoint code, unsigned int ctype);
+  int    (*get_ctype_code_range)(int ctype, int* nsb, int* nmb, RegCodePointRange* sbr[], RegCodePointRange* mbr[]);
+  UChar* (*left_adjust_char_head)(UChar* start, UChar* s);
+  int    (*is_allowed_reverse_match)(UChar* p, UChar* e);
+  int    (*get_all_fold_match_code)(RegCodePoint** codes);
+  int    (*get_fold_match_info)(UChar* p, UChar* end, EncFoldMatchInfo** info);
+} RegCharEncodingType;
+
+typedef RegCharEncodingType* RegCharEncoding;
+
+REG_EXTERN RegCharEncodingType RegEncodingASCII;
+REG_EXTERN RegCharEncodingType RegEncodingISO_8859_1;
+REG_EXTERN RegCharEncodingType RegEncodingISO_8859_15;
+REG_EXTERN RegCharEncodingType RegEncodingUTF8;
+REG_EXTERN RegCharEncodingType RegEncodingEUC_JP;
+REG_EXTERN RegCharEncodingType RegEncodingSJIS;
+REG_EXTERN RegCharEncodingType RegEncodingBIG5;
+
+#define REG_ENCODING_ASCII        (&RegEncodingASCII)
+#define REG_ENCODING_ISO_8859_1   (&RegEncodingISO_8859_1)
+#define REG_ENCODING_ISO_8859_15  (&RegEncodingISO_8859_15)
+#define REG_ENCODING_UTF8         (&RegEncodingUTF8)
+#define REG_ENCODING_EUC_JP       (&RegEncodingEUC_JP)
+#define REG_ENCODING_SJIS         (&RegEncodingSJIS)
+#define REG_ENCODING_BIG5         (&RegEncodingBIG5)
+
+/* for GNU regex API */
 #define MBCTYPE_ASCII         0
 #define MBCTYPE_EUC           1
 #define MBCTYPE_SJIS          2
 #define MBCTYPE_UTF8          3
 
-#define REGCODE_ASCII         REG_MBLEN_TABLE[MBCTYPE_ASCII]
-#define REGCODE_UTF8          REG_MBLEN_TABLE[MBCTYPE_UTF8]
-#define REGCODE_EUCJP         REG_MBLEN_TABLE[MBCTYPE_EUC]
-#define REGCODE_SJIS          REG_MBLEN_TABLE[MBCTYPE_SJIS]
-#define REGCODE_DEFAULT       REGCODE_ASCII
-
-REG_EXTERN const char REG_MBLEN_TABLE[][REG_CHAR_TABLE_SIZE];
 #endif /* else RUBY && M17N */
 
-REG_EXTERN RegCharEncoding RegDefaultCharEncoding;
+#define REG_ENCODING_UNDEF    ((RegCharEncoding )0)
 
-#if defined(RUBY_PLATFORM) && !defined(M17N_H)
+/* Don't use REGCODE_XXXX. (obsoleted) */
+#define REGCODE_UNDEF         REG_ENCODING_UNDEF
+#define REGCODE_ASCII         REG_ENCODING_ASCII
+#define REGCODE_UTF8          REG_ENCODING_UTF8
+#define REGCODE_EUCJP         REG_ENCODING_EUC_JP
+#define REGCODE_SJIS          REG_ENCODING_SJIS
+
+#if defined(RUBY_PLATFORM) && !defined(REG_RUBY_M17N)
+REG_EXTERN RegCharEncoding    EncDefaultCharEncoding;
 #undef ismbchar
 #define ismbchar(c)    (mbclen((c)) != 1)
-#define mbclen(c)      RegDefaultCharEncoding[(unsigned char )(c)]
+#define mbclen(c)      (EncDefaultCharEncoding->len_table[(unsigned char )(c)])
 #endif
 
 typedef unsigned int        RegOptionType;
 typedef unsigned char*      RegTransTableType;
 typedef unsigned int        RegDistance;
-typedef unsigned char       UChar;
 
 #define REG_OPTION_DEFAULT            REG_OPTION_NONE
 
@@ -170,7 +224,7 @@ REG_EXTERN RegSyntaxType*  RegDefaultSyntax;
 #define REG_SYN_OP_ESC_LPAREN_SUBEXP           (1<<13)  /* \(...\) */
 #define REG_SYN_OP_ESC_AZ_BUF_ANCHOR           (1<<14)  /* \A, \Z, \z */
 #define REG_SYN_OP_ESC_CAPITAL_G_BEGIN_ANCHOR  (1<<15)  /* \G     */
-#define REG_SYN_OP_BACK_REF                    (1<<16)  /* \num   */
+#define REG_SYN_OP_DECIMAL_BACKREF             (1<<16)  /* \num   */
 #define REG_SYN_OP_BRACKET_CC                  (1<<17)  /* [...]  */
 #define REG_SYN_OP_ESC_W_WORD                  (1<<18)  /* \w, \W */
 #define REG_SYN_OP_ESC_LTGT_WORD_BEGIN_END     (1<<19)  /* \<. \> */
@@ -416,11 +470,11 @@ void regex_set_warn_func P_((RegWarnFunc f));
 REG_EXTERN
 void regex_set_verb_warn_func P_((RegWarnFunc f));
 REG_EXTERN
-int regex_new P_((regex_t**, UChar* pattern, UChar* pattern_end, RegOptionType option, RegCharEncoding code, RegSyntaxType* syntax, RegErrorInfo* einfo));
+int regex_new P_((regex_t**, UChar* pattern, UChar* pattern_end, RegOptionType option, RegCharEncoding enc, RegSyntaxType* syntax, RegErrorInfo* einfo));
 REG_EXTERN
 void regex_free P_((regex_t*));
 REG_EXTERN
-int regex_recompile P_((regex_t*, UChar* pattern, UChar* pattern_end, RegOptionType option, RegCharEncoding code, RegSyntaxType* syntax, RegErrorInfo* einfo));
+int regex_recompile P_((regex_t*, UChar* pattern, UChar* pattern_end, RegOptionType option, RegCharEncoding enc, RegSyntaxType* syntax, RegErrorInfo* einfo));
 REG_EXTERN
 int regex_search P_((regex_t*, UChar* str, UChar* end, UChar* start, UChar* range, RegRegion* region, RegOptionType option));
 REG_EXTERN
@@ -450,15 +504,6 @@ REG_EXTERN
 RegOptionType regex_get_options P_((regex_t* reg));
 REG_EXTERN
 RegSyntaxType* regex_get_syntax P_((regex_t* reg));
-
-REG_EXTERN
-UChar* regex_get_prev_char_head P_((RegCharEncoding code, UChar* start, UChar* s));
-REG_EXTERN
-UChar* regex_get_left_adjust_char_head P_((RegCharEncoding code, UChar* start, UChar* s));
-REG_EXTERN
-UChar* regex_get_right_adjust_char_head P_((RegCharEncoding code, UChar* start, UChar* s));
-REG_EXTERN
-void regex_set_default_trans_table P_((UChar* table));
 REG_EXTERN
 int regex_set_default_syntax P_((RegSyntaxType* syntax));
 REG_EXTERN
@@ -470,6 +515,19 @@ int regex_end P_((void));
 REG_EXTERN
 const char* regex_version P_((void));
 
+/* encoding API */
+REG_EXTERN
+UChar* enc_get_prev_char_head P_((RegCharEncoding enc, UChar* start, UChar* s));
+REG_EXTERN
+UChar* enc_get_left_adjust_char_head P_((RegCharEncoding enc, UChar* start, UChar* s));
+REG_EXTERN
+UChar* enc_get_right_adjust_char_head P_((RegCharEncoding enc, UChar* start, UChar* s));
+
+/* obsoleted API */
+#define regex_get_prev_char_head         enc_get_prev_char_head
+#define regex_get_left_adjust_char_head  enc_get_left_adjust_char_head
+#define regex_get_right_adjust_char_head enc_get_right_adjust_char_head
+
 
 /* GNU regex API */
 #ifdef REG_RUBY_M17N
